<?php
/**
 * Plugin Name: Active Analytics
 * Plugin URI: https://getbutterfly.com/wordpress-plugins/active-analytics/
 * Description: Active Analytics is a WordPress-tailored analytics plugin, privacy-enhanced, lightweight solution for tracking what really matters: real time activity, most visited pages, best referrers and day-to-day/month-to-month users/pageviews statistics.
 * Author: 20script
 * Author URI: https://www.20script.ir
 * Version: 2.5.1
 *
 * Active Analytics
 * Copyright (C) 2014-2023 Ciprian Popescu (getbutterfly@gmail.com)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

define( 'WPAA_VERSION', '2.5.1' );

add_filter( 'plugin_action_links', 'wpaa_plugin_action_links', 10, 2 );

include 'includes/updater.php';

include 'includes/settings-statistics.php';
include 'includes/settings-traffic.php';
include 'includes/settings-views.php';

require_once 'includes/chart-overview.php';
require_once 'includes/functions.php';
require_once 'includes/client.php';
require_once 'includes/settings.php';

function wpaa_plugin_action_links( $links, $file ) {
    if ( (string) $file === (string) plugin_basename( __FILE__ ) ) {
        $settings_link = '<a href="' . admin_url( 'options-general.php?page=wpaa' ) . '">' . __( 'Settings', 'active-analytics' ) . '</a>';

        array_unshift( $links, $settings_link );
    }

    return $links;
}

function wpaa_init() {
    load_plugin_textdomain( 'active-analytics', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
}
add_action( 'plugins_loaded', 'wpaa_init' );

function wpaa_admin_enqueue_scripts() {
    wp_enqueue_style( 'thin-ui', plugins_url( '/assets/css/thin-ui.css', __FILE__ ), [], '2.1.1' );
    wp_enqueue_style( 'wpaa', plugins_url( '/assets/css/wpaa.css', __FILE__ ), [], WPAA_VERSION );

    wp_register_script( 'wpaa-chartjs', plugins_url( '/assets/js/chart.umd.min.js', __FILE__ ), [], '4.3.2', true );
    //wp_register_script( 'chartjs', 'https://cdnjs.cloudflare.com/ajax/libs/Chart.js/4.4.0/chart.umd.min.js', [], '4.4.0', true );

    wp_register_script( 'wpaa-charts', plugins_url( '/assets/js/wpaa-charts.js', __FILE__ ), [ 'wpaa-chartjs' ], WPAA_VERSION, true );
    wp_add_inline_script(
        'wpaa-charts',
        'const sa_var = ' . json_encode(
            [
                'ajaxurl' => admin_url( 'admin-ajax.php' ),
            ]
        ),
        'before'
    );
}

add_action( 'admin_enqueue_scripts', 'wpaa_admin_enqueue_scripts' );

function wpaa_enqueue_scripts() {
    global $post;

    if ( ! $post ) {
        $post_id = 0;
    } else {
        $post_id = $post->ID;
    }

    wp_enqueue_script( 'wpaa', plugins_url( '/assets/js/wpaa.min.js', __FILE__ ), [], WPAA_VERSION, true );

    $sa_var_referrer = isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : '';

    wp_add_inline_script(
        'wpaa',
        'const sa_var = ' . json_encode(
            [
                'ajaxurl'  => admin_url( 'admin-ajax.php' ),
                'post_id'  => $post_id,
                'referrer' => $sa_var_referrer,
            ]
        ),
        'before'
    );
}

add_action( 'wp_enqueue_scripts', 'wpaa_enqueue_scripts' );

function wpaa_substr( $str, $x, $y = 0 ) {
    if ( (int) $y === 0 ) {
        $y = strlen( $str ) - $x;
    }

    return substr( $str, $x, $y );
}

function wpaa_menu_links() {
    global $wpdb;

    $table_name = $wpdb->prefix . 'wpsa';

    if ( $wpdb->get_var( "SHOW TABLES LIKE '$table_name'" ) != $table_name ) {
        wpaa_create_table();
    }

    add_options_page(
        __( 'Active Analytics Settings', 'active-analytics' ),
        __( 'Active Analytics', 'active-analytics' ),
        'manage_options',
        'wpaa',
        'wpaa_settings'
    );
}

add_action( 'admin_menu', 'wpaa_menu_links', 10 );





add_action( 'wpaa_autodelete_cron_hook', 'wpaa_autodelete_cron' );

if ( ! function_exists( 'wpaa_autodelete_cron_schedule' ) ) {
    function wpaa_autodelete_cron_schedule() {
        if ( ! wp_next_scheduled( 'wpaa_autodelete_cron_hook' ) ) {
            wp_schedule_event( time(), 'daily', 'wpaa_autodelete_cron_hook' );
        }
    }
}

add_action( 'wp', 'wpaa_autodelete_cron_schedule' );

if ( ! function_exists( 'wpaa_autodelete_cron' ) ) {
    function wpaa_autodelete_cron() {
        global $wpdb;

        $table_name = $wpdb->prefix . 'wpsa';

        if ( (string) get_option( 'wpaa_autodelete' ) !== '' ) {
            $t = gmdate( 'Ymd', strtotime( '-' . get_option( 'wpaa_autodelete' ) ) );
            $results = $wpdb->query( "DELETE FROM " . $table_name . " WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') < '" . $t . "'" );
        }
    }
}



function wpaa_action_main() {
    global $wpdb;

    wp_enqueue_script( 'wpaa-chartjs' );
    wp_enqueue_script( 'wpaa-charts' );

    $table_name = $wpdb->prefix . 'wpsa';

    $old_version = get_option( 'wpaa_db_version', '1.0.0' );
    $new_version = '1.0.4';

    if ( ! ( version_compare( $old_version, $new_version ) >= 0 ) ) {
        $wpdb->query( "ALTER TABLE $table_name CHANGE session_id session_id VARCHAR(64) NULL DEFAULT NULL" );
        $wpdb->query( "ALTER TABLE $table_name ADD INDEX idx_session_id (session_id)" );
        $wpdb->query( "ALTER TABLE $table_name CHANGE agent agent VARCHAR(16) NULL DEFAULT NULL" );
        $wpdb->query( "ALTER TABLE $table_name CHANGE ip ip VARCHAR(64) NULL DEFAULT NULL" );
        $wpdb->query( "ALTER TABLE $table_name CHANGE timestamp timestamp INT NULL DEFAULT NULL" );

        $wpdb->query( "ALTER TABLE $table_name CHANGE urlrequested urlrequested VARCHAR(255) NULL DEFAULT NULL" );
        $wpdb->query( "ALTER TABLE $table_name CHANGE referrer referrer VARCHAR(255) NULL DEFAULT NULL" );

        update_option( 'wpaa_db_version', '1.0.4' );

        echo '<div class="updated"><p><strong>' . __( 'Active Analytics database structure has been updated!', 'active-analytics' ) . '</strong></p></div>';
    }

    $lastmonth = wpaa_lastmonth();
    $thismonth = gmdate( 'Ym', current_time( 'timestamp' ) );
    $yesterday = gmdate( 'Ymd', current_time( 'timestamp' ) - 86400 );
    $today     = gmdate( 'Ymd', current_time( 'timestamp' ) );
    $tlm[0]    = wpaa_substr( $lastmonth, 0, 4 );
    $tlm[1]    = wpaa_substr( $lastmonth, 4, 2 );

    echo '<h3>' . __( 'Welcome to Active Analytics!', 'active-analytics' ) . '</h3>

    <p>' . __( 'The dashboard shows you a general overview, last hits, last pages and referrers. Check the individual pages for more statistics.', 'active-analytics' ) . '</p>';

    echo '<p style="font-size: 18px;">';
        $to_time   = current_time( 'timestamp' );
        $from_time = strtotime( '-3 minutes', $to_time );
        $qry       = $wpdb->get_results( $wpdb->prepare( "SELECT count(DISTINCT(ip)) as visitors FROM $table_name WHERE timestamp BETWEEN %s AND %s;", $from_time, $to_time ) );

        echo '<b>' . $qry[0]->visitors . '</b> ' . __( 'unique users online', 'active-analytics' ) . ', ';

        $to_time   = current_time( 'timestamp' );
        $from_time = strtotime( '-30 minutes', $to_time );
        $qry       = $wpdb->get_results( $wpdb->prepare( "SELECT count(DISTINCT(ip)) as visitors FROM $table_name WHERE timestamp BETWEEN %s AND %s;", $from_time, $to_time ) );

        echo '<b>' . $qry[0]->visitors . '</b> ' . __( 'unique users in last 30 minutes', 'active-analytics' ) . '.';
    echo '</p>';

    echo '<table class="widefat no-border">
        <thead>
            <tr>
                <th scope="col"></th>
                <th scope="col">' . __( 'Total', 'active-analytics' ) . '</th>
                <th scope="col">' . __( 'Last month', 'active-analytics' ) . '<br><small>' . gmdate( 'M, Y', gmmktime( 0, 0, 0, $tlm[1], 1, $tlm[0] ) ) . '</small></th>
                <th scope="col">' . __( 'This month', 'active-analytics' ) . '<br><small>' . gmdate( 'M, Y', current_time( 'timestamp' ) ) . '</small></th>
                <th scope="col">' . __( 'Yesterday', 'active-analytics' ) . '<br><small>' . gmdate( 'd M, Y', current_time( 'timestamp' ) - 86400 ) . '</small></th>
                <th scope="col">' . __( 'Today', 'active-analytics' ) . '<br><small>' . gmdate( 'd M, Y', current_time( 'timestamp' ) ) . '</small></th>
            </tr>
        </thead>
        <tbody>';

            /**
             * Visitors
             */
            echo '<tr>
                <td><div class="wpaa-indicator--colour wpaa-indicator--colour-primary"></div>' . __( 'Visitors', 'active-analytics' ) . '</td>';

                // Total
                $qry_total = $wpdb->get_row( "SELECT count(DISTINCT ip) AS visitors FROM $table_name" );
                echo '<td>' . number_format( $qry_total->visitors ) . '</td>';

                // Last month
                $qry_lmonth = $wpdb->get_row( "SELECT count(DISTINCT ip) AS visitors FROM $table_name WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') LIKE '" . sanitize_text_field( $lastmonth ) . "%'" );
                echo '<td>' . number_format( $qry_lmonth->visitors ) . '</td>';

                // This month
                $qry_tmonth = $wpdb->get_row( "SELECT count(DISTINCT ip) AS visitors FROM $table_name WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') LIKE '" . sanitize_text_field( $thismonth ) . "%'" );

                $qry_tmonth->change = '';

                if ( (int) $qry_lmonth->visitors !== 0 ) {
                    $pc = round( 100 * ( $qry_tmonth->visitors / $qry_lmonth->visitors ) - 100, 1 );

                    if ( $pc >= 0 ) {
                        $pc = '+' . $pc;
                    }

                    $qry_tmonth->change = ' <code>(' . $pc . '%)</code>';
                }
                echo '<td>' . number_format( $qry_tmonth->visitors ) . $qry_tmonth->change . '</td>';

                // Yesterday
                $qry_y = $wpdb->get_row("SELECT count(DISTINCT ip) AS visitors
                    FROM $table_name
                    WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') = '" . sanitize_text_field($yesterday) . "'");
                echo '<td>' . number_format($qry_y->visitors) . '</td>';
        
                // Today
                $qry_t = $wpdb->get_row("SELECT count(DISTINCT ip) AS visitors
                    FROM $table_name
                    WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') = '" . sanitize_text_field($today) . "'");
                echo '<td>' . number_format($qry_t->visitors) . '</td>
            </tr>';

            /**
             * Pageviews
             */
            echo '<tr>
                <td><div class="wpaa-indicator--colour wpaa-indicator--colour-secondary"></div>' . __( 'Pageviews', 'active-analytics' ) . '</td>';

                // Total
                $qry_total       = $wpdb->get_row( "SELECT COUNT(FROM_UNIXTIME(timestamp, '%Y%m%d')) AS pageview FROM $table_name" );
                $pageviews_total = $qry_total->pageview;
                echo '<td>' . number_format( $pageviews_total ) . '</td>';

                // Last month
                $qry_lmonth           = $wpdb->get_row( "SELECT COUNT(FROM_UNIXTIME(timestamp, '%Y%m%d')) AS pageview FROM $table_name WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') LIKE '" . sanitize_text_field( $lastmonth ) . "%'" );
                $pageviews_last_month = $qry_lmonth->pageview;
                echo '<td>' . number_format( $pageviews_last_month ) . '</td>';

                // This month
                $qry_tmonth           = $wpdb->get_row( "SELECT COUNT(FROM_UNIXTIME(timestamp, '%Y%m%d')) AS pageview FROM $table_name WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') LIKE '" . sanitize_text_field( $thismonth ) . "%'" );
                $pageviews_this_month = $qry_tmonth->pageview;

                $qry_tmonth->change = '';

                if ( $pageviews_last_month > 0 ) {
                    $pc = round( 100 * ( $pageviews_this_month / $pageviews_last_month ) - 100, 1 );

                    if ( $pc >= 0 ) {
                        $pc = '+' . $pc;
                    }
                    $qry_tmonth->change = ' <code>(' . $pc . '%)</code>';
                }
                echo '<td>' . number_format( $pageviews_this_month ) . $qry_tmonth->change . '</td>';

                // Yesterday
                $qry_y = $wpdb->get_row("SELECT count(FROM_UNIXTIME(timestamp, '%Y%m%d')) as pageview
                    FROM $table_name
                    WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') = '" . sanitize_text_field($yesterday) . "'");
                echo '<td>' . number_format($qry_y->pageview) . '</td>';

                // Today
                $qry_t = $wpdb->get_row("SELECT count(FROM_UNIXTIME(timestamp, '%Y%m%d')) as pageview
                    FROM $table_name
                    WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') = '" . sanitize_text_field($today) . "'");
                echo '<td>' . number_format($qry_t->pageview) . '</td>
            </tr>';

            /**
             * Sessions
             *
             * Alternative: Use a subquery
             * Instead of calculating the distinct count directly on the entire table, you can use a subquery to reduce the number of rows processed.
             * The subquery retrieves the distinct session IDs, and the outer query counts the rows returned by the subquery.
             *
             * SELECT COUNT(*) AS total FROM ( SELECT DISTINCT session_id FROM bt_wpsa ) AS subquery;
             */

            // Total
            $qry_total      = $wpdb->get_row( "SELECT COUNT(DISTINCT(session_id)) AS session FROM $table_name" );
            $sessions_total = $qry_total->session;

            if ( $sessions_total > 0 ) {
                echo '<tr>
                    <td><div class="wpaa-indicator--colour wpaa-indicator--colour-tertiary"></div>' . __('Sessions', 'active-analytics') . '</td>';


                    echo '<td>' . number_format( $sessions_total ) . ' | ' . number_format( $pageviews_total / $sessions_total, 2 ) . ' <span class="dashicons dashicons-editor-help" style="color:#888888" title="Pageviews per session"></span></td>';

                    // Last month
                    $qry_lmonth = $wpdb->get_row("SELECT COUNT(DISTINCT(session_id)) AS session
                        FROM $table_name
                        WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') LIKE '" . sanitize_text_field($lastmonth) . "%'");
                    echo '<td>' . number_format($qry_lmonth->session) . '</td>';

                    // This month
                    $qry_tmonth = $wpdb->get_row("SELECT COUNT(DISTINCT(session_id)) as session
                        FROM $table_name
                        WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') LIKE '" . sanitize_text_field($thismonth) . "%'");

                    $qry_tmonth->change = '';

                    if ($qry_lmonth->session <> 0) {
                        $pc = round(100 * ($qry_tmonth->session / $qry_lmonth->session) - 100, 1);

                        if ($pc >= 0) {
                            $pc = '+' . $pc;
                        }
                        $qry_tmonth->change = ' <code>(' . $pc . '%)</code>';
                    }
                    echo '<td>' . number_format($qry_tmonth->session) . $qry_tmonth->change . '</td>';

                    // Yesterday
                    $qry_y = $wpdb->get_row("SELECT COUNT(DISTINCT(session_id)) as session
                        FROM $table_name
                        WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') = '" . sanitize_text_field($yesterday) . "'");
                    echo '<td>' . number_format($qry_y->session) . '</td>';

                    // Today
                    $qry_t = $wpdb->get_row("SELECT COUNT(DISTINCT(session_id)) as session
                        FROM $table_name
                        WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') = '" . sanitize_text_field($today) . "'");
                    echo '<td>' . number_format($qry_t->session) . '</td>
                </tr>';
            }

        echo '</tbody>
    </table>

    <br>';

    /**
     * Graphs
     */
    echo '<div class="wpaa-chart" id="wpaa-chart--overview">
        <div class="wpaa-spinner-wrap"><div>A</div></div>
        Loading overview...
    </div>';

    /**
     * Acquisition
     *
     * Last referrers columns (all time, last 30 days, today)
     */
    echo '<h2>' . __( 'Traffic Acquisition', 'active-analytics' ) . '</h2>';

    echo '<div class="thin-ui-grid thin-ui-grid--nowrap">';

        echo '<div class="thin-ui-col col--inner">';
            wpaa_table_referrers( __( 'Top Referrers (All Time)', 'active-analytics' ), 0 );
        echo '</div>';
        echo '<div class="thin-ui-col col--inner">';
            wpaa_table_referrers( __( 'Top Referrers (Last 30 Days)', 'active-analytics' ), 30 );
        echo '</div>';
        echo '<div class="thin-ui-col col--inner">';
            wpaa_table_referrers( __( 'Top Referrers (Today)', 'active-analytics' ), 1 );
        echo '</div>';

    echo '</div>';
    //

    $wpaa_lasthits_limit = ( (int) get_option( 'wpaa_lasthits_limit' ) > 0 ) ? (int) get_option( 'wpaa_lasthits_limit' ) : 100;
    $querylimit          = 'LIMIT ' . $wpaa_lasthits_limit;

    /**
     * Last Hits Table
     */
    echo '<h2>' . __( 'Last Hits', 'active-analytics' ) . '</h2>

    <div class="thin-ui-grid thin-ui-grid--nowrap">
        <div class="thin-ui-col col--inner">
            <table width="100%" cellpadding="8" cellspacing="0" class="last-hits-table--all">
                <thead>
                    <tr>
                        <th scope="col" style="width:20%">' . __( 'Time', 'active-analytics' ) . '</th>
                        <th scope="col" style="width:5%">' . __( 'Device', 'active-analytics' ) . '</th>
                        <th scope="col" style="width:75%">' . __( 'Page/Referrer', 'active-analytics' ) . '</th>
                    </tr>
                </thead>
                <tbody>';

                    $results = $wpdb->get_results( "SELECT timestamp, urlrequested, referrer, agent FROM $table_name WHERE agent <> '' ORDER BY id DESC $querylimit" );

                    foreach ( $results as $row ) {
                        $url_requested = trailingslashit( $row->urlrequested );

                        $result = $row->agent;

                        $result = ( $result === 'mobile' ) ? '<span class="dashicons dashicons-smartphone"></span>' : '<span class="dashicons dashicons-desktop"></span>';

                        echo '<tr data-ua="' . $row->agent . '">
                            <td style="white-space:nowrap">' . gmdate( 'F j, Y <b>H:i:s</b> ', $row->timestamp ) . '</td>
                            <td>' . $result . '</td>
                            <td>
                                <a href="' . $url_requested . '">' . $url_requested . '</a>';

                                if ( (string) $row->referrer !== '' ) {
                                    echo '<br><svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 377 377" class="wpaa-icon wpaa-icon-padded wpaa-icon-12"><path fill="var(--wpaa-ui-nav)" d="m275 216-22 21 45 45H52V0H22v312h276l-44 44 21 21 80-80z"></path></svg> <small title="' . $row->referrer . '" style="color:var(--wpaa-ui-nav)">' . wpaa_abbreviate( $row->referrer, 80 ) . '</small>';
                                }

                            echo '</td>
                        </tr>';
                    }

                echo '</tbody>
            </table>
        </div>
    </div>';
}



// New UI
add_action( 'wp_ajax_wpaa_chart_overview', 'wpaa_chart_overview' );
add_action( 'wp_ajax_wpaa_chart_dashboard', 'wpaa_chart_dashboard' );



function wpaa_action_engagement() {
    global $wpdb;

    wp_enqueue_script( 'wpaa-chartjs' );
    wp_enqueue_script( 'wpaa-charts' );

    $table_name = $wpdb->prefix . 'wpsa';

    echo '<h2>' . __( 'Engagement', 'active-analytics' ) . '</h2>';

    // Top content today
    $top_content_today_query = $wpdb->get_results(
        "SELECT 
            DATE_FORMAT(FROM_UNIXTIME(timestamp), '%Y-%m-%d') AS day,
            SUBSTRING_INDEX(urlrequested, '?', 1) AS formatted_url,
            COUNT(*) AS url_count
        FROM $table_name
        GROUP BY day, formatted_url
        ORDER BY day DESC, url_count DESC
        LIMIT 10;"
    );

    // Top content this month
    $top_content_this_month_query = $wpdb->get_results(
        "SELECT 
            DATE_FORMAT(FROM_UNIXTIME(timestamp), '%Y-%m') AS month,
            SUBSTRING_INDEX(urlrequested, '?', 1) AS formatted_url,
            COUNT(*) AS url_count
        FROM $table_name
        GROUP BY month, formatted_url
        ORDER BY month DESC, url_count DESC
        LIMIT 10;"
    );

    echo '<div class="thin-ui-grid">
        <div class="thin-ui-col col--inner">
            <h2>Top Content (Today)</h2>

            <table class="wpaa-table--details wpaa-table--details-links" cellpadding="0" cellspacing="0">
                <thead>
                    <tr>
                        <th class="wide">URL</th>
                        <th class="narrow" style="text-align:right">Pageviews</th>
                    </tr>
                </thead>
                <tbody>';

                foreach ( $top_content_today_query as $top_content ) {
                    echo '<tr>
                        <td>
                            <div class="wpaa-table--link">
                                <a href="' . $top_content->formatted_url . '" target="_blank">' . $top_content->formatted_url . ' <svg class="wpaa--icon" height="12px" width="12px" viewBox="0 0 12 12"><path fill="#7f8c8d" d="M5.35355 7.38909L10 2.74264V6H11V1H6V2H9.32843L4.64645 6.68198L5.35355 7.38909ZM1 11H10V8H9V10H2V3H4V2H1V11Z" fill-rule="evenodd"></path></svg></a>
                            </div>
                        </td>
                        <td style="text-align:right" class="wpaa-value--number">' .
                            number_format( $top_content->url_count ) .
                        '</td>
                    </tr>';
                }

                echo '</tbody>
            </table>
        </div>
        <div class="thin-ui-col col--inner">
            <h2>Top Content (This Month)</h2>

            <table class="wpaa-table--details wpaa-table--details-links" cellpadding="0" cellspacing="0">
                <thead>
                    <tr>
                        <th class="wide">URL</th>
                        <th class="narrow" style="text-align:right">Pageviews</th>
                    </tr>
                </thead>
                <tbody>';

                foreach ( $top_content_this_month_query as $top_content ) {
                    echo '<tr>
                        <td>
                            <div class="wpaa-table--link">
                                <a href="' . $top_content->formatted_url . '" target="_blank">' . $top_content->formatted_url . ' <svg class="wpaa--icon" height="12px" width="12px" viewBox="0 0 12 12"><path fill="#7f8c8d" d="M5.35355 7.38909L10 2.74264V6H11V1H6V2H9.32843L4.64645 6.68198L5.35355 7.38909ZM1 11H10V8H9V10H2V3H4V2H1V11Z" fill-rule="evenodd"></path></svg></a>
                            </div>
                        </td>
                        <td style="text-align:right" class="wpaa-value--number">' .
                            number_format( $top_content->url_count ) .
                        '</td>
                    </tr>';
                }

                echo '</tbody>
            </table>
        </div>
    </div>
    <div class="thin-ui-grid">
        <div class="thin-ui-col col--inner">';
            // Top Pages
            wpaa_value_table( 'urlrequested', __( 'Top Content (All Time)', ' active-analytics' ), 10, '', 'urlrequested', '' );
        echo '</div>
    </div>';


    // Devices
    $mobile_count  = $wpdb->get_var( "SELECT COUNT(agent) AS mobile_count FROM $table_name WHERE agent = 'mobile' GROUP BY agent;" );
    $desktop_count = $wpdb->get_var( "SELECT COUNT(agent) AS desktop_count FROM $table_name WHERE agent = 'desktop' GROUP BY agent;" );

    echo '<div class="thin-ui-grid thin-ui-grid--nowrap" style="margin: 16px 0">
        <div class="thin-ui-col col--inner">
            <canvas id="chartjs-device-values" height="128" data-sets="' . $mobile_count . '|' . $desktop_count . '"></canvas>
        </div>
    </div>';
    //

    echo '<div class="thin-ui-grid thin-ui-grid--nowrap">
        <div class="thin-ui-col col--inner">';
            // Top Days - Unique visitors
            wpaa_value_table( 'date', __( 'Top Days - Unique visitors', 'active-analytics' ), 10, 'distinct', 'ip', '' );
        echo '</div>
        <div class="thin-ui-col col--inner">';
            // Top Days - Pageviews
            wpaa_value_table( 'date', __( 'Top Days - Pageviews', 'active-analytics' ), 10, '', 'urlrequested', '' );
        echo '</div>
    </div>';
}



function wpaa_action_spy() {
    global $wpdb;

    $table_name = $wpdb->prefix . 'wpsa';

    $limit = 20;

    if ( isset( $_GET['pn'] ) ) {
        $page = (int) sanitize_text_field( $_GET['pn'] );

        if ( (int) $page <= 0 ) {
            $page = 1;
        }
    } else {
        $page = 1;
    }

    $query       = $wpdb->get_results( "SELECT id FROM $table_name GROUP BY ip" );
    $total_rows  = $wpdb->num_rows;
    $num_pages   = $total_rows / $limit;
    $limit_value = ( $page * $limit ) - $limit;

    // Spy
    $today     = gmdate( 'Ymd', current_time( 'timestamp') );
    $yesterday = gmdate( 'Ymd', current_time( 'timestamp' ) - 86400 );

    echo '<h2>' . __( 'Spy', 'active-analytics' ) . '</h2>';

    $sql = "SELECT ip, agent FROM $table_name GROUP BY ip ORDER BY id DESC LIMIT $limit_value, $limit";
    $qry = $wpdb->get_results( $sql );
    ?>

    <div id="wpaa-pagination">
        <?php _e( 'Pages:', 'active-analytics' ); ?>
        <?php
        if ( (int) $page === (int) ceil( $num_pages ) && (int) $page !== 1 ) {
            for ( $i = 1; $i <= ceil( $num_pages ) - 1; $i++ ) {
                if ( (int) $i > 0 ) {
                    echo '<a href="' . $_SERVER['SCRIPT_NAME'] . '?page=wpaa&tab=spy&pn=' . $i . '">' . $i . '</a> ';
                }
            }
        }

        if ( (int) $page === (int) ceil( $num_pages ) ) {
            $start_page = $page;
        } else {
            $start_page = 1;
        }

        for ( $i = $start_page; $i <= $page + 6; $i++ ) {
            if ( $i <= ceil( $num_pages ) ) {
                if ( (int) $i === (int) $page ) {
                    echo " [{$i}] ";
                } else {
                    echo '<a href="' . $_SERVER['SCRIPT_NAME'] . '?page=wpaa&tab=spy&pn=' . $i . '">' . $i . '</a> ';
                }
            }
        }
        ?>
    </div>

    <table id="mainspytab" class="wpaa-spy-table" name="mainspytab" width="100%" border="0" cellspacing="0" cellpadding="4">
        <?php
        foreach ( $qry as $rk ) {
            echo '<tr>
                <td colspan="2">
                    <details>
                        <summary><code>' . trim( $rk->ip ) . '</code> ' . __( 'More info', 'active-analytics' ) . '</summary>

                        <p><code>' . $rk->agent . '</code></p>
                        <p><small><code>' . gethostbyaddr( $rk->ip ) . '</code></small></p>
                    </details>
                </td>
            </tr>';

            $qry2 = $wpdb->get_results( "SELECT urlrequested, referrer, timestamp FROM $table_name WHERE ip = '" . $rk->ip . "' AND (FROM_UNIXTIME(timestamp, '%Y%m%d') BETWEEN '$yesterday' AND '$today') ORDER BY id DESC LIMIT 10" );

            foreach ( $qry2 as $details ) {
                echo '<tr>
                    <td valign="top" width="151">
                        <small><strong>' . gmdate( 'F j, Y H:i:s', $details->timestamp ) . '</strong></small>
                    </td>
                    <td>
                        <a href="' . $details->urlrequested . '" target="_blank">' . $details->urlrequested . '</a>';

                        if ( (string) $details->referrer !== '' && strpos( $details->referrer, get_option( 'home' ) ) === false ) {
                            echo '<br><small>' . __( 'arrived from', 'active-analytics' ) . ' <a href="' . $details->referrer . '" target="_blank">' . $details->referrer . '</a></small>';
                        }
                    echo '</td>
                </tr>';
            }
        }
        ?>
    </table>
    <?php
}



function wpaa_abbreviate($s, $c) {
    $res = '';

    if (strlen($s) > $c) {
        $res = '...';
    }

    return wpaa_substr($s, 0, $c) . $res;
}



function wpaa_get_table_size( $table ) {
    global $wpdb;

    $res = $wpdb->get_results( "SHOW TABLE STATUS LIKE '$table'" );

    foreach ( $res as $fstatus ) {
        $data_lenght = $fstatus->Data_length;
        $data_rows   = $fstatus->Rows;
    }

    return number_format( ( $data_lenght / 1024 / 1024 ), 2 ) . 'MB (' . $data_rows . ' records)';
}



function wpaa_value_table( $fld, $fldtitle, $limit = 0, $param = '', $queryfld = '', $exclude = '' ) {
    global $wpdb;

    $table_name = $wpdb->prefix . 'wpsa';

    if ( (string) $queryfld === '' ) {
        $queryfld = $fld;
    }

    echo '<h2>' . $fldtitle . '</h2>

    <table class="wpaa-table--details wpaa-table--details-links" cellpadding="0" cellspacing="0">
        <thead>
            <tr>
                <th class="wide">' . __('URL', 'active-analytics') . '</th>
                <th class="narrow" style="text-align:right">' . __('Pageviews', 'active-analytics') . '</th>
                <th class="wide">' . __('Percentage', 'active-analytics') . '</th>
            </tr>
        </thead>
        <tbody>';

            $rks = $wpdb->get_var("SELECT count($param $queryfld) as rks FROM $table_name WHERE 1=1 $exclude;");

            if ($rks > 0) {
                $should_format = false;
                $date_format   = get_option( 'date_format' );

                if ( (string) $fld === 'date' ) {
                    $should_format = true;

                    $fld = "FROM_UNIXTIME(timestamp, '%Y%m%d')";
                }

                $sql = "SELECT count($param $queryfld) as pageview, $fld FROM $table_name WHERE 1=1 $exclude GROUP BY $fld ORDER BY pageview DESC";

                if ($limit > 0) {
                    $sql = $sql . " LIMIT $limit";
                }

                $qry = $wpdb->get_results($sql);

                foreach ($qry as $rk) {
                    $pc = round(($rk->pageview * 100 / $rks), 1);
                    $uri = '';

                    if ((string) $fld === 'urlrequested') {
                        $rk->$fld = trailingslashit( $rk->$fld );
                        $uri = $rk->$fld;
                    } else {
                        $rk->$fld = wpaa_substr($rk->$fld, 0, 50);
                    }

                    echo '<tr>
                        <td>' . ( ( $should_format ) ? date( $date_format, strtotime( $rk->$fld ) ) : '<a href="' . $rk->$fld . '" target="_blank">' . $rk->$fld . ' <svg class="wpaa--icon" height="12px" width="12px" viewBox="0 0 12 12"><path fill="#7f8c8d" d="M5.35355 7.38909L10 2.74264V6H11V1H6V2H9.32843L4.64645 6.68198L5.35355 7.38909ZM1 11H10V8H9V10H2V3H4V2H1V11Z" fill-rule="evenodd"></path></svg></a>' );

                        echo '</td>
                        <td style="text-align:right" class="wpaa-value--number">' . number_format($rk->pageview) . '</td>
                        <td>
                            <div class="wpaa-graph-line" data-width="' . ( 100 * $pc / 100 ) . '">' . $pc . '%</div>
                        </td>
                    </tr>';
                }
            }

        echo '</tbody>
    </table>';
}



function wpaa_table_referrers( $title = '', $when = 0 ) {
    global $wpdb;

    $fld        = 'referrer';
    $limit      = ( (int) get_option( 'wpaa_referrer_limit' ) > 0 ) ? (int) get_option( 'wpaa_referrer_limit' ) : 10;
    $date_query = '';
    $table_name = $wpdb->prefix . 'wpsa';

    if ( (int) $when > 0 ) {
        $date_query = "AND FROM_UNIXTIME(timestamp, '%Y%m%d') BETWEEN DATE_SUB(NOW(), INTERVAL $when DAY) AND NOW()";
    }

    echo '<h2>' . $title . '</h2>';

    $rks = $wpdb->get_var( "SELECT count(referrer) as rks FROM $table_name WHERE referrer <> '' AND referrer NOT LIKE '%" . get_bloginfo( 'url' ) . "%' $date_query" );

    if ( $rks > 0 ) {
        $sql = "SELECT count(referrer) as pageview, $fld FROM $table_name WHERE referrer <> '' AND referrer NOT LIKE '%" . get_bloginfo( 'url' ) . "%' $date_query GROUP BY $fld ORDER BY pageview DESC LIMIT $limit";
        $qry = $wpdb->get_results( $sql );

        foreach ( $qry as $rk ) {
            $pc = round( ( $rk->pageview * 100 / $rks ), 1 );

            echo '<div style="word-break:break-word">
                <a href="' . $rk->$fld . '">' . $rk->$fld . '</a>
                <div class="wpaa-graph-line" data-width="' . ( 100 * $pc / 100 ) . '">' . $pc . '% (' . number_format( $rk->pageview ) . ' ' . __( 'visits', 'active-analytics' ) . ')</div>
            </div>';
        }
    }
}



function wpaa_lastmonth() {
    $ta = getdate(current_time('timestamp'));

    $year = (int) $ta['year'];
    $month = (int) $ta['mon'];

    // Go back 1 month
    $month = $month - 1;

    if ($month === 0) {
        // If this month is January, go back a year
        $year  = $year - 1;
        $month = 12;
    }

    // Return in format 'YYYYMM'
    return sprintf($year . '%02d', $month);
}



function wpaa_create_table() {
    global $wpdb;

    require_once ABSPATH . 'wp-admin/includes/upgrade.php';

    $table_name = $wpdb->prefix . 'wpsa';
    $sql_createtable = "CREATE TABLE " . $table_name . " (
        id int(11) NOT NULL AUTO_INCREMENT,
        ip varchar(64) DEFAULT NULL,
        urlrequested varchar(255) DEFAULT NULL,
        agent varchar(16) DEFAULT NULL,
        referrer varchar(255) DEFAULT NULL,
        timestamp int DEFAULT NULL,
        session_id varchar(64) DEFAULT NULL,
        UNIQUE KEY id (id)
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

    dbDelta( $sql_createtable );

    maybe_convert_table_to_utf8mb4( $table_name );

    $wpdb->query( "ALTER TABLE $table_name DROP date" );
    $wpdb->query( "ALTER TABLE $table_name DROP time" );

    add_option( 'wpaa_collectloggeduser', '' );
    add_option( 'wpaa_autodelete', '1 year' );
    add_option( 'wpaa_daysinoverviewgraph', 30 );
    add_option( 'wpaa_pageviews_key', '_aa_pageviews' );

    update_option( 'wpaa_db_version', '1.0.4' );
}

function wpaa_update_table() {
    global $wpdb;

    $old_version = get_option( 'wpaa_db_version', '1.0.0' );
    $new_version = '1.0.4';

    if ( ! ( version_compare( $old_version, $new_version ) < 0 ) ) {
        return false;
    }

    require_once ABSPATH . 'wp-admin/includes/upgrade.php';

    $table_name = $wpdb->prefix . 'wpsa';

    $wpdb->query( "ALTER TABLE $table_name ADD session_id VARCHAR(64)" );
    $wpdb->query( "ALTER TABLE $table_name CHANGE session_id session_id VARCHAR(64) NULL DEFAULT NULL" );
    $wpdb->query( "ALTER TABLE $table_name ADD INDEX idx_session_id (session_id)" );
    $wpdb->query( "ALTER TABLE $table_name CHANGE agent agent VARCHAR(16) NULL DEFAULT NULL" );
    $wpdb->query( "ALTER TABLE $table_name CHANGE ip ip VARCHAR(64) NULL DEFAULT NULL" );
    $wpdb->query( "ALTER TABLE $table_name CHANGE timestamp timestamp INT NULL DEFAULT NULL" );

    $wpdb->query( "ALTER TABLE $table_name CHANGE urlrequested urlrequested VARCHAR(255) NULL DEFAULT NULL" );
    $wpdb->query( "ALTER TABLE $table_name CHANGE referrer referrer VARCHAR(255) NULL DEFAULT NULL" );

    update_option( 'wpaa_db_version', '1.0.4' );
}

add_action( 'admin_init', 'wpaa_update_table' );



add_action( 'wp_ajax_wpaa_pageview', 'wpaa_append_client' );
add_action( 'wp_ajax_nopriv_wpaa_pageview', 'wpaa_append_client' );



register_activation_hook( __FILE__, 'wpaa_create_table' );



function wpaa_custom_dashboard_widgets() {
    wp_add_dashboard_widget('wpaa_dashboard_widget_overview', 'Active Analytics', 'wpaa_dashboard_widget_overview');
}

if ((string) get_option('wpaa_dashboard_widget') === 'checked') {
    add_action('wp_dashboard_setup', 'wpaa_custom_dashboard_widgets');
}

function wpaa_dashboard_widget_overview() {
    global $wpdb;

    wp_enqueue_script( 'wpaa-chartjs' );
    wp_enqueue_script( 'wpaa-charts' );

    $table_name = $wpdb->prefix . 'wpsa';

    $lastmonth = wpaa_lastmonth();
    $thismonth = gmdate( 'Ym', current_time( 'timestamp' ) );
    $yesterday = gmdate( 'Ymd', current_time( 'timestamp' ) - 86400 );
    $today     = gmdate( 'Ymd', current_time( 'timestamp' ) );
    $tlm[0]    = wpaa_substr( $lastmonth, 0, 4 );
    $tlm[1]    = wpaa_substr( $lastmonth, 4, 2 );

    echo '<div class="wpaa-chart" id="wpaa-chart--dashboard">
        <div class="wpaa-spinner-wrap"><div>A</div></div>
        Loading overview...
    </div>';

    echo '<p>';
        $to_time   = current_time( 'timestamp' );
        $from_time = strtotime( '-3 minutes', $to_time );
        $qry       = $wpdb->get_results( $wpdb->prepare( "SELECT count(DISTINCT(ip)) as visitors FROM $table_name WHERE timestamp BETWEEN %s AND %s;", $from_time, $to_time ) );

        echo '<b>' . $qry[0]->visitors . '</b> ' . __( 'unique users online', 'active-analytics' ) . ', ';

        $to_time   = current_time( 'timestamp' );
        $from_time = strtotime( '-30 minutes', $to_time );
        $qry       = $wpdb->get_results( $wpdb->prepare( "SELECT count(DISTINCT(ip)) as visitors FROM $table_name WHERE timestamp BETWEEN %s AND %s;", $from_time, $to_time ) );

        echo '<b>' . $qry[0]->visitors . '</b> ' . __( 'unique users in last 30 minutes', 'active-analytics' ) . '.';
    echo '</p>';

    echo '<table class="widefat no-border">
        <thead>
            <tr>
                <th scope="col"></th>
                <th scope="col">' . __( 'Last month', 'active-analytics' ) . '<br><small>' . gmdate( 'M', gmmktime( 0, 0, 0, $tlm[1], 1, $tlm[0] ) ) . '</small></th>
                <th scope="col">' . __( 'This month', 'active-analytics' ) . '<br><small>' . gmdate( 'M', current_time( 'timestamp' ) ) . '</small></th>
                <th scope="col">' . __( 'Yesterday', 'active-analytics' ) . '<br><small>' . gmdate( 'd M', current_time( 'timestamp' ) - 86400 ) . '</small></th>
                <th scope="col">' . __( 'Today', 'active-analytics' ) . '<br><small>' . gmdate( 'd M', current_time( 'timestamp' ) ) . '</small></th>
            </tr>
        </thead>
        <tbody>';

            /**
             * Visitors
             */
            echo '<tr>
                <td>' . __( 'Visitors', 'active-analytics' ) . '</td>';

                // Last month
                $qry_lmonth = $wpdb->get_row( "SELECT count(DISTINCT ip) AS visitors FROM $table_name WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') LIKE '" . sanitize_text_field( $lastmonth ) . "%'" );
                echo '<td>' . number_format( $qry_lmonth->visitors ) . '</td>';

                // This month
                $qry_tmonth = $wpdb->get_row( "SELECT count(DISTINCT ip) AS visitors FROM $table_name WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') LIKE '" . sanitize_text_field( $thismonth ) . "%'" );

                if ( $qry_lmonth->visitors <> 0 ) {
                    $pc = round( 100 * ( $qry_tmonth->visitors / $qry_lmonth->visitors ) - 100, 1 );

                    if ( $pc >= 0 ) {
                        $pc = '+' . $pc;
                    }

                    $qry_tmonth->change = '<br><code style="font-size: 11px;">' . $pc . '%</code>';
                }
                echo '<td>' . number_format( $qry_tmonth->visitors ) . $qry_tmonth->change . '</td>';

                // Yesterday
                $qry_y = $wpdb->get_row( "SELECT count(DISTINCT ip) AS visitors FROM $table_name WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') = '" . sanitize_text_field( $yesterday ) . "'" );
                echo '<td>' . number_format( $qry_y->visitors ) . '</td>';

                // Today
                $qry_t = $wpdb->get_row( "SELECT count(DISTINCT ip) AS visitors FROM $table_name WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') = '" . sanitize_text_field( $today ) . "'" );
                echo '<td>' . number_format( $qry_t->visitors ) . '</td>
            </tr>';

            /**
             * Pageviews
             */
            echo '<tr>
                <td>' . __( 'Pageviews', 'active-analytics' ) . '</td>';

                // Last month
                $qry_lmonth = $wpdb->get_row( "SELECT COUNT(FROM_UNIXTIME(timestamp, '%Y%m%d')) AS pageview FROM $table_name WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') LIKE '" . sanitize_text_field( $lastmonth ) . "%'" );
                echo '<td>' . number_format( $qry_lmonth->pageview ) . '</td>';

                // This month
                $qry_tmonth = $wpdb->get_row( "SELECT COUNT(FROM_UNIXTIME(timestamp, '%Y%m%d')) AS pageview FROM $table_name WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') LIKE '" . sanitize_text_field( $thismonth ) . "%'" );

                if ( $qry_lmonth->pageview <> 0 ) {
                    $pc = round( 100 * ( $qry_tmonth->pageview / $qry_lmonth->pageview ) - 100, 1 );

                    if ( $pc >= 0 ) {
                        $pc = '+' . $pc;
                    }

                    $qry_tmonth->change = '<br><code style="font-size: 11px;">' . $pc . '%</code>';
                }

                echo '<td>' . number_format( $qry_tmonth->pageview ) . $qry_tmonth->change . '</td>';

                // Yesterday
                $qry_y = $wpdb->get_row( "SELECT COUNT(FROM_UNIXTIME(timestamp, '%Y%m%d')) AS pageview FROM $table_name WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') = '" . sanitize_text_field( $yesterday ) . "'" );
                echo '<td>' . number_format( $qry_y->pageview ) . '</td>';

                // Today
                $qry_t = $wpdb->get_row( "SELECT COUNT(FROM_UNIXTIME(timestamp, '%Y%m%d')) AS pageview FROM $table_name WHERE FROM_UNIXTIME(timestamp, '%Y%m%d') = '" . sanitize_text_field( $today ) . "'" );
                echo '<td>' . number_format( $qry_t->pageview ) . '</td>
            </tr>
        </tbody>
    </table>';

    echo '<p>
        <a href="' . admin_url( 'options-general.php?page=wpaa' ) . '" class="button">' . __( 'View all stats', 'active-analytics' ) . '</a>
    </p>';
}
// 910
