/* global Chart, sa_var */
/* eslint no-undef: "error" */

window.aa_font_family = '-apple-system, BlinkMacSystemFont, "Segoe UI", Helvetica, Arial, sans-serif, "Segoe UI Emoji", "Apple Color Emoji"';

async function fetchTest(url) {
    let response = await fetch(url);
    let responseText = await response.text();

    document.getElementById('wpaa-chart--overview').innerHTML = responseText;
}

async function fetchTestDashboard(url) {
    let response = await fetch(url);
    let responseText = await response.text();

    document.getElementById('wpaa-chart--dashboard').innerHTML = responseText;
}



function addPagerToTables(tables, rowsPerPage = 10) {
    tables = typeof tables == 'string' ? document.querySelectorAll(tables) : tables;

    for (let table of tables) {
        addPagerToTable(table, rowsPerPage);
    }
}

function addPagerToTable(table, rowsPerPage = 10) {
    let tBodyRows = table.querySelectorAll('tBody tr');
    let numPages = Math.ceil(tBodyRows.length / rowsPerPage);

    let colCount = [].slice.call(table.querySelector('tr').cells).reduce((a, b) => a + parseInt(b.colSpan), 0);

    table.createTFoot().insertRow().innerHTML = `<td colspan=${colCount}><div class="nav"></div></td>`;

    if (numPages == 1) {
        return;
    }

    for (i = 0; i < numPages; i++) {
        let pageNum = i + 1;

        table.querySelector('.nav')
            .insertAdjacentHTML(
                'beforeend',
                `<a href="#" rel="${i}">${pageNum}</a> `
            );

    }

    changeToPage(table, 1, rowsPerPage);

    for (let navA of table.querySelectorAll('.nav a')) {
        navA.addEventListener('click', e => {
            e.preventDefault();

            changeToPage(table, parseInt(e.target.innerHTML), rowsPerPage);
        });
    }
}

function changeToPage(table, page, rowsPerPage) {
    let startItem = (page - 1) * rowsPerPage;
    let endItem = startItem + rowsPerPage;
    let navAs = table.querySelectorAll('.nav a');
    let tBodyRows = table.querySelectorAll('tbody tr');

    for (let nix = 0; nix < navAs.length; nix++) {
        if (nix == page - 1) {
            navAs[nix].classList.add('active');
        } else {
            navAs[nix].classList.remove('active');
        }

        for (let trix = 0; trix < tBodyRows.length; trix++) {
            tBodyRows[trix].style.display =
                (trix >= startItem && trix < endItem)
                    ? 'table-row'
                    : 'none';
        }
    }
}



document.addEventListener('DOMContentLoaded', () => {
    if (document.querySelector('.wpaa-graph-line')) {
        [].forEach.call(document.querySelectorAll('.wpaa-graph-line'), line => {
            var width = line.dataset.width;
            var inverse = 100 / width * 100;

            line.style.width = width + '%';

            // Set the background size to the inverse of the width
            line.style.backgroundSize = inverse + '% 100%';
        });
    }

    if (document.getElementById('wpaa-chart--overview')) {
        let wpaa_datastring = 'action=wpaa_chart_overview';

        (async () => {
            await fetchTest(`${sa_var.ajaxurl}?${wpaa_datastring}`);

            const ctx = document.getElementById('myChart');
            let dataset_labels = ctx.dataset.labels;
            let dataset_users = ctx.dataset.users;
            let dataset_pageviews = ctx.dataset.pageviews;
            let dataset_sessions = ctx.dataset.sessions;

            const myChart = new Chart(ctx, {
                type: "line",
                data: {
                    labels: dataset_labels.split(','),
                    datasets: [
                        {
                            label: 'Users',
                            data: dataset_users.split(','),
                            fill: true,
                            backgroundColor: 'rgb(88 0 255 / 25%)',
                            borderColor: '#5800FF',
                            borderWidth: 2,
                            pointRadius: 2,
                            pointHoverRadius: 2,
                            tension: 0.2
                        },
                        {
                            label: 'Sessions',
                            data: dataset_sessions.split(','),
                            fill: false,
                            backgroundColor: 'rgb(0 215 255 / 25%)',
                            borderColor: '#00D7FF',
                            borderWidth: 2,
                            pointRadius: 2,
                            pointHoverRadius: 2,
                            tension: 0.2
                        },
                        {
                            label: 'Pageviews',
                            data: dataset_pageviews.split(','),
                            fill: true,
                            backgroundColor: 'rgb(0 150 255 / 25%)',
                            borderColor: '#0096FF',
                            borderWidth: 2,
                            pointRadius: 2,
                            pointHoverRadius: 2,
                            tension: 0.2
                        }
                    ]
                },
                options: {
                    scales: {
                        y: {
                            beginAtZero: true,
                            ticks: {
                                font: {
                                    family: aa_font_family
                                }
                            }
                        },
                        x: {
                            ticks: {
                                maxRotation: 90,
                                minRotation: 90,
                                font: {
                                    family: aa_font_family
                                }
                            }
                        }
                    },
                    responsive: true,
                    plugins: {
                        tooltip: {
                            intersect: false,
                            mode: 'index',
                            titleFont: {
                                family: aa_font_family
                            },
                            bodyFont: {
                                family: aa_font_family
                            },
                            footerFont: {
                                family: aa_font_family
                            }
                        },
                        legend: {
                            position: 'top',
                            labels: {
                                font: {
                                    family: aa_font_family
                                }
                            }
                        },
                        title: {
                            display: false,
                            font: {
                                family: aa_font_family
                            }
                        }
                    }
                }
            });
        })();
    }

    if (document.getElementById('wpaa-chart--dashboard')) {
        let wpaa_datastring = 'action=wpaa_chart_dashboard';

        (async () => {
            await fetchTestDashboard(`${sa_var.ajaxurl}?${wpaa_datastring}`);

            const ctx = document.getElementById('myChart');
            let dataset_labels = ctx.dataset.labels;
            let dataset_users = ctx.dataset.users;
            let dataset_pageviews = ctx.dataset.pageviews;
            let dataset_sessions = ctx.dataset.sessions;

            const myChart = new Chart(ctx, {
                type: 'bar',
                data: {
                    labels: dataset_labels.split(','),
                    datasets: [
                        {
                            label: 'Users',
                            data: dataset_users.split(','),
                            fill: true,
                            backgroundColor: 'rgb(88 0 255 / 100%)',
                        },
                        {
                            label: 'Sessions',
                            data: dataset_sessions.split(','),
                            fill: false,
                            backgroundColor: 'rgb(0 215 255 / 100%)',
                        },
                        {
                            label: 'Pageviews',
                            data: dataset_pageviews.split(','),
                            fill: true,
                            backgroundColor: 'rgb(0 150 255 / 100%)',
                        }
                    ]
                },
                options: {
                    scales: {
                        y: {
                            beginAtZero: true,
                            ticks: {
                                font: {
                                    family: aa_font_family
                                }
                            }
                        },
                        x: {
                            stacked: true,
                            ticks: {
                                maxRotation: 90,
                                minRotation: 90,
                                font: {
                                    family: aa_font_family
                                }
                            }
                        }
                    },
                    responsive: true,
                    plugins: {
                        tooltip: {
                            intersect: false,
                            mode: 'index',
                            titleFont: {
                                family: aa_font_family
                            },
                            bodyFont: {
                                family: aa_font_family
                            },
                            footerFont: {
                                family: aa_font_family
                            }
                        },
                        legend: {
                            position: 'top',
                            labels: {
                                font: {
                                    family: aa_font_family
                                },
                                usePointStyle: true,
                                pointStyle: 'rectRounded'
                            }
                        },
                        title: {
                            display: false,
                            font: {
                                family: aa_font_family
                            }
                        }
                    }
                }
            });
        })();
    }

    if (document.getElementById('chartjs-device-values')) {
        /**
         * Get chart holder element
         */
        const barValuesChartHolder = document.getElementById('chartjs-device-values');
        const fontFamily = '-apple-system, BlinkMacSystemFont, "Segoe UI", Helvetica, Arial, sans-serif, "Segoe UI Emoji", "Apple Color Emoji"';

        /**
         * Get chart holder data attributes (labels and datasets)
         */
        let datasets = barValuesChartHolder.dataset.sets.split('|'),
            ctxBarValues = barValuesChartHolder.getContext('2d');

        /**
         * Build the speed factors chart
         */
        let chartBarValues = new Chart(ctxBarValues, {
            //type: 'horizontalBar',
            type: 'bar',

            data: {
                labels: ['Devices'],
                datasets: [{
                    label: 'Mobile',
                    data: [datasets[0]],
                    backgroundColor: '#5800FF'
                },
                {
                    label: 'Desktop',
                    data: [datasets[1]],
                    backgroundColor: '#0096FF'
                }]
            },

            options: {
                indexAxis: 'y',
                plugins: {
                    tooltips: {
                        titleFontSize: 15,
                        titleFontFamily: fontFamily,
                        bodyFontFamily: fontFamily,
                        bodyFontSize: 14,
                        intersect: true,
                        mode: 'index',
                        enabled: true
                    },
                    scales: {
                        xAxes: [{
                            ticks: {
                                beginAtZero: true,
                                fontSize: 12
                            },
                            scaleLabel: {
                                display: false
                            },
                            gridLines: {
                            },
                            stacked: true
                        }],
                        yAxes: [{
                            gridLines: {
                                display: false,
                                color: '#fff',
                                zeroLineColor: '#fff',
                                zeroLineWidth: 0
                            },
                            ticks: {
                                fontSize: 12
                            },
                            stacked: false
                        }]
                    },
                    legend: {
                        display: true,
                        position: 'top',
                        align: 'start',
                    },
                }
            }
        });
    }


    if (document.querySelector('.top-pages-table--all')) {
        addPagerToTables('.top-pages-table--all', 10);
    }

    if (document.querySelector('.top-posts-table--all')) {
        addPagerToTables('.top-posts-table--all', 10);
    }

    if (document.querySelector('.last-hits-table--all')) {
        addPagerToTables('.last-hits-table--all', 10);
    }

    if (document.querySelector('.link-rankings-table--all')) {
        addPagerToTables('.link-rankings-table--all', 10);
    }

    if (document.querySelector('.page-links-table--all')) {
        addPagerToTables('.page-links-table--all', 10);
    }
});
