/**
 * Store a pageview
 *
 * Stores a pageview's IP address and location URL.
 *
 * @param string url      URL
 * @param string referrer Referrer
 */
async function wpaa_pageview(url, referrer) {
    // If the user agent is a crawler
    if (navigator.userAgent.match(/bot|crawl|spider/i)) {
        return false;
    }

    let current_ts = Math.floor(Date.now() / 1000), // Current timestamp in seconds
        device = 'desktop'; // Get device type // alpha

    // If session-id local storage item does not exist
    // If expiry-ts local storage item does not exist
    // If session time has expired
    if (localStorage.getItem('session-id') === null || localStorage.getItem('expiry-ts') === null || current_ts > parseInt(localStorage.getItem('expiry-ts'), 10)) {
        // Create a new session with a random ID and store
        // Random ID here is a concat of rand(11111111, 99999999) and timestamp
        let session_id = [(Math.floor(Math.random() * (99999999 - 11111111 + 1)) + 11111111), Date.now()].join('');

        localStorage.setItem('session-id', session_id);
    }

    // Set expiry to current timestamp + 30 minutes
    localStorage.setItem('expiry-ts', current_ts + 1800);

    if ('NavigatorUAData' in window) {
        if (navigator.userAgentData.mobile) {
            device = 'mobile';
        }
    } else {
        // Break user agent down
        const userAgent = navigator.userAgent.toLowerCase();

        if (userAgent.match(/mobile|tablet|android|iphone|ipad/i)) {
            device = 'mobile';
        }
    }
    //

    // Create a new URLSearchParams object
    let params = new URLSearchParams();
    params.append('action', 'wpaa_pageview');
    params.append('pv_id', sa_var.post_id);
    params.append('pv_url', url);
    params.append('pv_referrer', encodeURIComponent(referrer));
    params.append('pv_device', device);
    params.append('pv_session_id', localStorage.getItem('session-id'));

    // Use the toString() method to get the query string
    let datastring = params.toString();

    // Make the HTTP request
    const response = await fetch(`${sa_var.ajaxurl}?${datastring}`, {
        method: 'GET',
        credentials: 'same-origin',
        headers: {
            'Content-Type': 'text/plain'
        }
    });
}



document.addEventListener('DOMContentLoaded', () => {
    /**
     * Store a pageview
     *
     */
    wpaa_pageview(location.href, sa_var.referrer);
});
