<?php
function wpaa_action_traffic() {
    global $wpdb;

    wp_enqueue_script( 'wpaa-chartjs' );
    wp_enqueue_script( 'wpaa-charts' );

    echo '<h2>' . __( 'Traffic', 'active-analytics' ) . '</h2>';

    echo '<p>' . __( 'This is a monthly view of each of your website\'s pages and posts.', 'active-analytics' ) . '</p>';

    $selected = ( isset( $_POST['page_id'] ) && (int) $_POST['page_id'] > 0 ) ? (int) $_POST['page_id'] : 0;

    $args = [
        'depth'                 => 0,
        'child_of'              => 0,
        'selected'              => $selected,
        'echo'                  => 0,
        'name'                  => 'page_id',
        'id'                    => '',
        'class'                 => '',
        'show_option_none'      => 'Select a page...',
        'show_option_no_change' => '',
        'option_none_value'     => 0,
        'value_field'           => 'ID',
    ];

    echo '<form method="post">
        <p>' . wp_dropdown_pages( $args ) . '</p>

        <p><input type="submit" class="button button-secondary" name="go" value="Go"></p>
    </form>

    <p><small>OR</small></p>

    <form method="post">
        <p>' . wpaa_generate_post_select( 'page_id', 'post', $selected ) . '</p>

        <p><input type="submit" class="button button-secondary" name="go" value="Go"></p>
    </form>';

    $table_name = $wpdb->prefix . 'wpsa';

    $page_id = $selected;

    if ( $page_id > 0 ) {
        $lp_url = get_permalink( $page_id );

        echo '<h2>Performance Chart</h2>
        <h3>Landing Page: ' . get_the_title( $page_id ) . '</h3>';

        $lp_pvs          = [];
        $lp_months       = [];
        $lp_month_names  = [];
        $lp_distinct_pvs = [];

        // Get all pageviews/impressions
        $nuuk_fragments = $wpdb->get_results( "SELECT COUNT(ip) AS pv_all, COUNT(DISTINCT ip) AS pv_distinct_all, FROM_UNIXTIME(timestamp, '%m') AS pv_month, FROM_UNIXTIME(timestamp, '%Y') AS pv_year FROM $table_name WHERE urlrequested LIKE '$lp_url%' GROUP BY FROM_UNIXTIME(timestamp, '%m'), FROM_UNIXTIME(timestamp, '%Y') ORDER BY FROM_UNIXTIME(timestamp, '%Y') DESC, FROM_UNIXTIME(timestamp, '%m') DESC LIMIT 24" );

        foreach ( $nuuk_fragments as $row_fragment ) {
            $lp_pvs[]          = $row_fragment->pv_all;
            $lp_distinct_pvs[] = $row_fragment->pv_distinct_all;
            $lp_months[]       = $row_fragment->pv_month;
        }

        // Reverse arrays
        $lp_pvs          = array_reverse( $lp_pvs );
        $lp_distinct_pvs = array_reverse( $lp_distinct_pvs );
        $lp_months       = array_reverse( $lp_months );

        // Convert month numbers to month names
        foreach ( $lp_months as $lp_month ) {
            $date_object      = DateTime::createFromFormat( '!m', $lp_month );
            $lp_month_names[] = $date_object->format( 'F' );
        }
        ?>

        <div class="wpaa-chart" id="wpaa-chart--traffic">
            <canvas id="pv-chart-<?php echo $page_id; ?>" style="width:100%;height:400px;"></canvas>
        </div>

        <script>
        document.addEventListener('DOMContentLoaded', () => {
            ctx = document.getElementById('pv-chart-<?php echo $page_id; ?>').getContext('2d');
            myChart = new Chart(ctx, {
                type: 'line',
                data: {
                    labels: <?php echo json_encode( $lp_month_names ); ?>,
                    datasets: [
                        {
                            label: 'Users',
                            data: <?php echo json_encode( $lp_distinct_pvs ); ?>,
                            fill: true,
                            backgroundColor: "rgb(88 0 255 / 25%)",
                            borderColor: "#5800FF",
                            borderWidth: 2,
                            pointRadius: 2,
                            pointHoverRadius: 2,
                            tension: 0.2
                        },
                        {
                            label: 'Pageviews',
                            data: <?php echo json_encode( $lp_pvs ); ?>,
                            fill: true,
                            backgroundColor: "rgb(0 150 255 / 25%)",
                            borderColor: "#0096FF",
                            borderWidth: 2,
                            pointRadius: 2,
                            pointHoverRadius: 2,
                            tension: 0.2
                        }
                    ]
                },
                options: {
                    scales: {
                        y: {
                            beginAtZero: true
                        },
                        x: {
                            ticks: {
                                maxRotation: 90,
                                minRotation: 90
                            }
                        }
                    },
                    responsive: true,
                    plugins: {
                        tooltip: {
                            intersect: false,
                            mode: 'index'
                        },
                        legend: {
                            position: 'top',
                            labels: {
                                font: {
                                    family: '-apple-system, BlinkMacSystemFont, "Segoe UI", Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji"'
                                }
                            }
                        },
                        title: {
                            display: false,
                            text: 'Performance Chart',
                            font: {
                                family: '-apple-system, BlinkMacSystemFont, "Segoe UI", Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji"'
                            }
                        }
                    }
                }
            });
        });
        </script>
        <?php
    }
}
