<?php
function wpaa_settings() {
    $tab     = ( filter_has_var( INPUT_GET, 'tab' ) ) ? filter_input( INPUT_GET, 'tab' ) : 'dashboard';
    $section = 'admin.php?page=wpaa&amp;tab=';
    ?>
    <div class="wrap wpaa-ui">
        <h1><?php _e( 'Active Analytics', 'active-analytics' ); ?></h1>

        <h2 class="nav-tab-wrapper">
            <a href="<?php echo $section; ?>dashboard" class="nav-tab <?php echo $tab === 'dashboard' ? 'nav-tab-active' : ''; ?>"><?php _e( 'Dashboard', 'active-analytics' ); ?></a>
            <a href="<?php echo $section; ?>details" class="nav-tab <?php echo $tab === 'details' ? 'nav-tab-active' : ''; ?>"><?php _e( 'Engagement', 'active-analytics' ); ?></a>
            <a href="<?php echo $section; ?>traffic" class="nav-tab <?php echo $tab === 'traffic' ? 'nav-tab-active' : ''; ?>"><?php _e( 'Traffic', 'active-analytics' ); ?></a>
            <a href="<?php echo $section; ?>views" class="nav-tab <?php echo $tab === 'views' ? 'nav-tab-active' : ''; ?>"><?php _e( 'Pageviews', 'active-analytics' ); ?></a>
            <a href="<?php echo $section; ?>internal" class="nav-tab <?php echo $tab === 'internal' ? 'nav-tab-active' : ''; ?>"><?php _e( 'Internal Links', 'active-analytics' ); ?></a>
            <a href="<?php echo $section; ?>spy" class="nav-tab <?php echo $tab === 'spy' ? 'nav-tab-active' : ''; ?>"><?php _e( 'Spy', 'active-analytics' ); ?></a>
            <a href="<?php echo $section; ?>statistics" class="nav-tab <?php echo $tab === 'statistics' ? 'nav-tab-active' : ''; ?>"><?php _e( 'Statistics', 'active-analytics' ); ?></a>
            <a href="<?php echo $section; ?>settings" class="nav-tab <?php echo $tab === 'settings' ? 'nav-tab-active' : ''; ?>"><?php _e( 'General Settings', 'active-analytics' ); ?></a>
        </h2>

        <?php
        if ( $tab === 'dashboard' ) {
            wpaa_action_main();
        } elseif ( $tab === 'details' ) {
            wpaa_action_engagement();
        } elseif ( $tab === 'views' ) {
            wpaa_action_views();
        } elseif ( $tab === 'internal' ) {
            include 'settings-internal.php';
        } elseif ( $tab === 'spy' ) {
            wpaa_action_spy();
        } elseif ( $tab === 'statistics' ) {
            wpaa_action_statistics();
        } elseif ( $tab === 'traffic' ) {
            wpaa_action_traffic();
        } elseif ( $tab === 'settings' ) {
            wpaa_action_options();
        }
        ?>
    </div>
    <?php
}

function wpaa_action_options() {
    if ( isset( $_POST['wpaa_save'] ) ) {
        update_option( 'wpaa_collectloggeduser', sanitize_text_field( $_POST['wpaa_collectloggeduser'] ) );
        update_option( 'wpaa_dashboard_widget', sanitize_text_field( $_POST['wpaa_dashboard_widget'] ) );
        update_option( 'wpaa_autodelete', sanitize_text_field( $_POST['wpaa_autodelete'] ) );

        $wpaa_pageviews_key = ( (string) $_POST['wpaa_pageviews_key'] !== '' ) ? sanitize_text_field( $_POST['wpaa_pageviews_key'] ) : '_aa_pageviews';
        update_option( 'wpaa_pageviews_key', $wpaa_pageviews_key );

        if ( (string) $_POST['wpaa_autodelete'] !== '' ) {
            wpaa_autodelete_cron_schedule();
        } else {
            wp_clear_scheduled_hook( 'wpaa_autodelete_cron_hook' );
        }

        update_option( 'wpaa_daysinoverviewgraph', (int) sanitize_text_field( $_POST['wpaa_daysinoverviewgraph'] ) );

        update_option( 'wpaa_referrer_limit', (int) sanitize_text_field( $_POST['wpaa_referrer_limit'] ) );
        update_option( 'wpaa_lasthits_limit', (int) sanitize_text_field( $_POST['wpaa_lasthits_limit'] ) );
        update_option( 'wpaa_internal_threshold', (int) sanitize_text_field( $_POST['wpaa_internal_threshold'] ) );

        update_option( 'wpaa_collect_post_views', (int) sanitize_text_field( $_POST['wpaa_collect_post_views'] ), true );

        delete_option( 'wpaa_ua_parser' );
        delete_option( 'wpaa_tracking' );

        wpaa_create_table();

        echo '<div class="updated notice is-dismissible"><p>' . __( 'Settings updated successfully!', 'active-analytics' ) . '</p></div>';
    } elseif ( isset( $_POST['wpaa_delete_pageview_key'] ) ) {
        delete_post_meta_by_key( get_option( 'wpaa_pageviews_key' ) );
    }
    ?>
    <h2><?php _e( 'General Settings', 'active-analytics' ); ?></h2>

    <form method="post">
        <table class="form-table">
            <tbody>
                <tr>
                    <th scope="row"><label><?php _e( 'Display Features', 'active-analytics' ); ?></label></th>
                    <td>
                        <p>
                            <input type="checkbox" name="wpaa_dashboard_widget" value="checked" <?php echo get_option( 'wpaa_dashboard_widget' ); ?>> 
                            <label for="wpaa_dashboard_widget"><?php _e( 'Display overview widget on Dashboard (may slow down the Dashboard)', 'active-analytics' ); ?></label>
                        </p>
                        <p>
                            <input type="number" name="wpaa_referrer_limit" min="5" max="50" step="5" value="<?php echo get_option( 'wpaa_referrer_limit' ); ?>">
                            <label for="wpaa_referrer_limit"><?php _e( 'Number of top referrers to show on Dashboard (default is 10)', 'active-analytics' ); ?></label>
                        </p>
                        <p>
                            <input type="number" name="wpaa_lasthits_limit" min="10" max="500" step="10" value="<?php echo get_option( 'wpaa_lasthits_limit' ); ?>">
                            <label for="wpaa_lasthits_limit"><?php _e( 'Number of last hits to show on Dashboard (default is 100)', 'active-analytics' ); ?></label>
                        </p>
                        <p>
                            <input type="number" name="wpaa_internal_threshold" min="1" max="1000" step="1" value="<?php echo get_option( 'wpaa_internal_threshold' ); ?>">
                            <label for="wpaa_internal_threshold"><?php _e( 'Number of internal links to consider for Internal Link charts (default is 5)', 'active-analytics' ); ?></label>
                        </p>
                    </td>
                </tr>
                <tr>
                    <th scope="row"><label><?php _e( 'Analytics Features', 'active-analytics' ); ?></label></th>
                    <td>
                        <p>
                            <input type="checkbox" name="wpaa_collectloggeduser" value="checked" <?php echo get_option( 'wpaa_collectloggeduser' ); ?>> 
                            <label for="wpaa_collectloggeduser"><?php _e( 'Collect data about logged in users', 'active-analytics' ); ?></label>
                        </p>
                        <p>
                            <input type="checkbox" name="wpaa_collect_post_views" value="1" <?php checked( 1, (int) get_option( 'wpaa_collect_post_views' ) ); ?>> 
                            <label for="wpaa_collect_post_views"><?php _e( 'Count pageviews as a custom post meta key', 'active-analytics' ); ?></label>
                        </p>
                    </td>
                </tr>
                <tr>
                    <th scope="row"><label for="wpaa_pageview_key"><?php _e( 'Pageview Count', 'active-analytics' ); ?></label></th>
                    <td>
                        <p>
                            <input name="wpaa_pageviews_key" id="wpaa_pageviews_key" type="text" class="regular-text" placeholder="_aa_pageviews" value="<?php echo get_option( 'wpaa_pageviews_key' ); ?>">
                            <label for="wpaa_pageviews_key">Pageviews meta key</label>
                            <br><small>Use this key to migrate your old pageviews or to display the count in your theme or plugin.</small>
                        </p>
                    </td>
                </tr>
                <tr>
                    <th scope="row"><label><?php _e( 'Overview', 'active-analytics' ); ?></label></th>
                    <td>
                        <p>
                            <select name="wpaa_daysinoverviewgraph">
                                <option value="7" <?php selected( (int) get_option( 'wpaa_daysinoverviewgraph' ), 7 ); ?>><?php _e( '7 (1 Week)', 'active-analytics' ); ?></option>
                                <option value="14" <?php selected( (int) get_option( 'wpaa_daysinoverviewgraph' ), 14 ); ?>><?php _e( '14 (2 Weeks)', 'active-analytics' ); ?></option>
                                <option value="30" <?php selected( (int) get_option( 'wpaa_daysinoverviewgraph' ), 30 ); ?>><?php _e( '30 (1 Month)', 'active-analytics' ); ?></option>
                                <option value="60" <?php selected( (int) get_option( 'wpaa_daysinoverviewgraph' ), 60 ); ?>><?php _e( '60 (2 Months)', 'active-analytics' ); ?></option>
                                <option value="90" <?php selected( (int) get_option( 'wpaa_daysinoverviewgraph' ), 90 ); ?>><?php _e( '90 (3 Months)', 'active-analytics' ); ?></option>
                            </select>
                            <label for="wpaa_daysinoverviewgraph"><?php _e( 'Days in Overview Graph', 'active-analytics' ); ?></label>
                        </p>
                    </td>
                </tr>

                <tr>
                    <th scope="row"><label for="wpaa_autodelete"><?php _e( 'Data Retention', 'active-analytics' ); ?></label></th>
                    <td>
                        <p>
                            Automatically delete visits older than 
                            <select name="wpaa_autodelete">
                                <option value="" <?php selected( (string) get_option( 'wpaa_autodelete' ), '' ); ?>><?php _e( 'Never Delete', 'active-analytics' ); ?></option>
                                <option value="1 month" <?php selected( (string) get_option( 'wpaa_autodelete' ), '1 month' ); ?>><?php _e( '1 month', 'active-analytics' ); ?></option>
                                <option value="3 months" <?php selected( (string) get_option( 'wpaa_autodelete' ), '3 months' ); ?>><?php _e( '3 months', 'active-analytics' ); ?></option>
                                <option value="6 months" <?php selected( (string) get_option( 'wpaa_autodelete' ), '6 months' ); ?>><?php _e( '6 months', 'active-analytics' ); ?></option>
                                <option value="1 year" <?php selected( (string) get_option( 'wpaa_autodelete' ), '1 year' ); ?>><?php _e( '1 year', 'active-analytics' ); ?></option>
                                <option value="2 years" <?php selected( (string) get_option( 'wpaa_autodelete' ), '2 years' ); ?>><?php _e( '2 years', 'active-analytics' ); ?></option>
                            </select>
                        </p>
                    </td>
                </tr>

                <tr>
                    <th scope="row"></th>
                    <td>
                        <p>
                            <input type="submit" name="wpaa_save" class="button button-primary" value="<?php _e( 'Save Changes', 'active-analytics' ); ?>">
                            <input type="submit" name="wpaa_delete_pageview_key" class="button button-secondary" value="<?php _e( 'Delete Pageview Key', 'active-analytics' ); ?>">
                        </p>
                    </td>
                </tr>
            </tbody>
        </table>
    </form>
    <?php
}
